% --------------------------------------------------
%
% Main file for computation of visualization of
% the results shown in Figures 9, 10, 16, and 17
% in the paper. 
%
% The topic is likelihood-free inference of the full
% Ricker model using Bayesian optimization. 
%
% --------------------------------------------------

clear all

% run startup(); manually to set the path
%startup();

figFolder = '../../figures/Ricker';
PAPER = 0;
if PAPER
    % for results in the paper
    matFolder = '../../matfiles/Ricker/paper_results'; 
else
    matFolder = '../../matfiles/Ricker';
end
%

mkdir(figFolder)

allTasks = {'simulations',...              % runs BOLFI
            'plot_evolution',...           % visualizes the progress of BOLFI as more data is acquired
            'comp_posterior',...           % computates the model-based posterior
            'make_main_figures',...        % makes Figures 9 and 10 in the paper
            'synLik_MCMC_R_to_matlab',...  % makes the R MCMC results more easily accessible in matlab
            'analyze_MCMC_runs',...        % makes Figures 16 and 17 in the paper
           };

% Choose task
taskNb = 1;
task = allTasks{taskNb}

% Task-dependent settings
if strcmp(task,'simulations')
    iterStart = 1;
    iterEnd = 1;
    N = 500;
    reqDataPoints_in = 90;
end

if strcmp(task,'plot_evolution')
  
    if PAPER
        myFile = [matFolder '/point_N500_slack05_log1_reqDataPoint90_run02_1']; 
    else
        % example
        myFile = [matFolder '/point_N500_slack05_log1_reqDataPoint90_run01_1']; 
    end
    
end

if strcmp(task,'comp_posterior')
    
    if PAPER
        myFile = [matFolder '/point_N500_slack05_log1_reqDataPoint90_run02_1'];
        myRound = [1:7];
    else
        % example 
        myFile = [matFolder '/point_N500_slack05_log1_reqDataPoint90_run01_1'];
        myRound = [1:7];
    end

end
    
if strcmp(task,'make_main_figures')
   
    if PAPER
        myFile = ['point_N500_slack05_log1_reqDataPoint90_run02_1']; 
        myRound = [3 4 6 7];
        maxRound = 7;
        EXPORT = 0;
    else
        % example
        myFile = ['point_N500_slack05_log1_reqDataPoint90_run01_1']; 
        myRound = [3 4 6 7];
        maxRound = 7;
        EXPORT = 0;
    end

end
    
if strcmp(task,'synLik_MCMC_R_to_matlab')
    loadDir = '../../matfiles/Ricker/paper_results';
end

if strcmp(task,'analyze_MCMC_runs')
    EXPORT = 0;
end
    

switch task
    
  case 'simulations'
        
    %% Data
    rng('default')
    Yobs = load([matFolder '/obs_Y.txt']);
    thetaObs = [3.8 0.3 10];
    
    %seed = rng;
    %thetaObs = 3.8;
    %Yobs = sampleDataDistribution(thetaObs,seed,1);
    %Yobs = Yobs{1};
    
    %save([matFolder '/obs_Y.txt'],'-ascii','-double','Yobs')
    
    rng('shuffle')
    for kIter = iterStart:iterEnd
           
        saveFileTag = ['run01_' num2str(kIter)];
        % init
        options = setOptions(saveFileTag,N,reqDataPoints_in);
        obj = @(theta)(objective(theta,[],N,Yobs));
    
        evidence = init(options,obj);
    
        if(0)
            figure;quickPlot(evidence,options,1,1);
            figure;quickPlot(evidence,options,1,2);
            figure;quickPlot(evidence,options,1,3);   
        end
        x = evidence.X(1:evidence.nX,:);
        f = evidence.f(1:evidence.nX);
        
        % if options are adjusted, check effect on initialization
        % options = setOptions(saveFileTag,N,reqDataPoints_in);
        % evidence = init(options,x,f);
        if(0)
            figure;quickPlot(evidence,options,1,1);
            figure;quickPlot(evidence,options,1,2);
            figure;quickPlot(evidence,options,1,3);   
        end
        
        % run
        [thetaHat evidence options] = fastInference(obj,options,evidence);

    end
    
    case 'plot_evolution'

      load([myFile '_round0.mat'],'options');    
      slashIndex = strfind(options.IO.saveFile,'/');
      if ~isempty(slashIndex)
          slashIndex = slashIndex(end);
      else
          slashIndex = 0;
      end
      fileID = options.IO.saveFile(slashIndex+1:end);
      mypath = [matFolder '/' fileID];
  
      LOGSCALE = 1;
      
      for k=0:options.ACQ.maxRounds
          
          load([mypath '_round' num2str(k) '.mat'],'evidence','options')
          figNb=figure('position', [800, 200, 900, 900]);
          clf;
          for dim1=1:options.GP.dim
              subplot(3,3,3*(dim1-1)+dim1)
              quickPlot(evidence,options,LOGSCALE,dim1);                      
          end
          for dim1=1:options.GP.dim
              for dim2=dim1+1:options.GP.dim
                  subplot(3,3,3*(dim1-1)+dim2)
                  quickPlot2D(evidence,options,[dim1 dim2],LOGSCALE)
              end
          end
          pause(0.5)
      end
     
       
  case 'comp_posterior'
    
    for kRound = 1:length(myRound)
        resFileBase = [myFile '_round' num2str(myRound(kRound))];

        resFile = [resFileBase '.mat'];
        load(resFile,'evidence','options');
        
        likOptions.bandwidth = [1/sqrt(2)]; % worked with -logL
        likOptions.method = 'logGP-synlik';
        samplingOptions.method = 'SMC';
        samplingOptions.nSamples = 25000;
        samplingOptions.priorPdf = @(u)(priorPdf(u,options.ACQ.optim.bounds));
        samplingOptions.nGenerations = 3;
        samplingOptions.fileName = [resFileBase '_posterior.mat'];
        
        sample_posterior(evidence,options,likOptions,samplingOptions);
    end

    
  case 'make_main_figures'
    
    thetaObs = [3.8 0.3 10];
    
    myFilePath = [matFolder '/' myFile]; 
          
    %% estimated regression function
    % Figure 9
    
    myLim = [2 12];
    for round=1:maxRound
        if EXPORT
            fileName = [figFolder '/' myFile '_regression_round_' num2str(round)];
        else
            fileName = '';
        end
        res = load([myFilePath '_round' num2str(round) '.mat']);
        plotLearnedModel(res.evidence,res.options,fileName,myLim);
        fprintf(['Round ' num2str(round) ...
                 ' nb of points: ' num2str(res.evidence.nX) ...
                 ' thetaHat = ' num2str(res.evidence.thetaHat(round+1,:)) '\n'])
    end
    
    %% marginal posteriors
    % Figure 10
    
    % grid 
    nGrid = 200;
    res = load([myFilePath '_round1.mat']);
    bounds = res.options.ACQ.optim.bounds;
    
    % marginals, posterior mean and credibility intervals
    nRounds = length(myRound);
    cc=flipud(gray(nRounds+2));
    %cc=hsv(nRounds+2);
    myMean = zeros(3,nRounds);
    myLCI = zeros(3,nRounds);
    myUCI = zeros(3,nRounds);
    myThetaJ = zeros(3,nRounds);

    figNbs=zeros(3,1);
    figNbs(1)=figure;
    plot(bounds(1,:),1/diff(bounds(1,:))*[1 1],'color',cc(2,:))
    figNbs(2)=figure;
    plot(bounds(2,:),1/diff(bounds(2,:))*[1 1],'color',cc(2,:))
    figNbs(3)=figure;
    plot(bounds(3,:),1/diff(bounds(3,:))*[1 1],'color',cc(2,:))

    cc = cc(3:end,:);
    acqData = zeros(1,nRounds);
    for counter=1:length(myRound)
        round = myRound(counter);
        res = load([myFilePath '_round' num2str(round) '.mat']);
        acqData(counter) = res.evidence.nX;
        myThetaJ(:,counter) = getThetaJ(res.evidence);
        
        loadFile = [myFilePath '_round' num2str(round) '_posterior.mat'];
        [all_post_mean CI all_hist lower_edges] = getMarginals(loadFile,bounds,[0.025 0.975],nGrid);
        myMean(:,counter) = all_post_mean;
        myLCI(:,counter) = CI(:,1);
        myUCI(:,counter) = CI(:,2);
        plotMarginals(all_post_mean, CI, all_hist, lower_edges,figNbs,cc(counter,:));
    end
    
    nSims = res.options.ACQ.nInitPoints+cumsum(res.options.ACQ.maxAcqPerRound);
    nSims = nSims*res.options.OBJ.N;
    nSims = nSims(myRound);   
      
    % MCMC solution
    load([matFolder '/MCMC_nReps500.mat'],'res500s','res500s_b');
    nStart = 25000;
    nBins = 80;
    
    para = 1;
    figure(figNbs(para))
    h=plot_MCMC_res(res500s.theta(para,:),nStart,lower_edges(para,:),nBins,figNbs(para),'bv',[],'kde'); set(h,'linestyle','--');   
    set(gca,'xlim',[3 4.8])
    plot(myMean(para,end),0,'ks')
    plot(thetaObs(para),0,'g*')
    plot(mean(res500s.theta(para,nStart:end)),0,'bd')    
    
    para = 2;
    figure(figNbs(para))
    h=plot_MCMC_res(res500s.theta(para,:),nStart,lower_edges(para,:),nBins,figNbs(para),'bv',0.05,'kde'); set(h,'linestyle','--');   
    h=plot_MCMC_res(res500s_b.theta(para,:),nStart,lower_edges(para,:),nBins,figNbs(para),'bv',0.05,'kde'); set(h,'linestyle','--');      
    set(gca,'xlim',[-0.01 0.85])  
    plot(myMean(para,end),0,'ks')
    plot(thetaObs(para),0,'g*')
    plot(mean(res500s.theta(para,nStart:end)),0,'bd')
    plot(mean(res500s_b.theta(para,nStart:end)),0,'bd')
   
    para = 3;
    figure(figNbs(para))
    h=plot_MCMC_res(res500s.theta(para,:),nStart,lower_edges(para,:),nBins,figNbs(para),'bv',[],'kde'); set(h,'linestyle','--');
    set(gca,'xlim',[6 13])  
    plot(myMean(para,end),0,'ks')
    plot(thetaObs(para),0,'g*')
    plot(mean(res500s.theta(para,nStart:end)),0,'bd')
    
    chainLength = size(res500s.theta,2);
    nSimsMCMC_500 = 500*(chainLength);
    str{1} = 'prior';
    for k=1:length(nSims)
        ratio = nSimsMCMC_500/nSims(k);
        str{k+1} = char(['evals: ' num2str(acqData(k))],['ratio: ' num2str(ratio,3)]);
    end
    str{length(nSims)+2} = char(['evals: ' num2str(chainLength)],'(MCMC)');
    
    figure(figNbs(1))
    legend(str)
    set(gca,'xlim',[bounds(1,1) bounds(1,2)])
    xlabel('log r')
    ylabel('estimated posterior pdf')      
    if EXPORT
        fileName = [figFolder '/' myFile '_model_posterior_pdf_logr'];
        myexportfig(fileName)
    end
    
    figure(figNbs(2))
    legend(str)
    set(gca,'xlim',[bounds(2,1) bounds(2,2)])
    xlabel('\sigma')
    ylabel('estimated posterior pdf') 
    if EXPORT
        fileName = [figFolder '/' myFile '_model_posterior_pdf_sigma'];
        myexportfig(fileName)
    end
   
    figure(figNbs(3))
    legend(str)
    set(gca,'xlim',[bounds(3,1) bounds(3,2)])
    xlabel('\phi')
    ylabel('estimated posterior pdf') 
    if EXPORT
        fileName = [figFolder '/' myFile '_model_posterior_pdf_phi'];
        myexportfig(fileName)
    end
   
  case 'synLik_MCMC_R_to_matlab'
       
    % run1 : n.reps <- 500 theta <- c(3.8,0.3,10),   prop.sd <- c(.02,0.05,.05),  n.mc <- 10000
    % run2 : n.reps <- 500 theta <- c(3.8,0.3,10),   prop.sd <- c(.02,0.01,.05),  n.mc <- 10000, accept: 29.75%
    % run3 : n.reps <- 500 theta <- c(3.8,0.3,10),   prop.sd <- c(.02,0.01,.05),  n.mc <- 50000, accept: 28.978%
    % run3b : n.reps <- 500 theta <- c(3.8,0.3,10),   prop.sd <- c(.02,0.02,.05),  n.mc <- 50000, accept: 17.3%
    
    % run6 : n.reps <- 500 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.1,.05),  n.mc <- 100000, accept : 10.1%
    % run7 : n.reps <- 500 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.05,.05),  n.mc <- 100000, accept: 12.0%
    % run8 : n.reps <- 500 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.01,.05),  n.mc <- 100000, accept: 22.1%
    % run21: like run8 (500reps) but with set.seed(100)  instead of set.seed(3), accept: 24.3 %
    
    % run4 : n.reps <- 1000 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.1,.05),  n.mc <- 100000, accept: 10.7%
    % run5 : n.reps <- 1000 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.05,.05),  n.mc <- 100000, accept: 15.8%
    % run9 : n.reps <- 1000 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.01,.05),  n.mc <- 100000, accept: 27.1 %
    % run22: like run9 (1000reps) but with set.seed(100)  instead of set.seed(3), accept: 26.0 

    % run10 : n.reps <- 5000 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.1,.05),  n.mc <- 100000, accept: 9%
    % run11 : n.reps <- 5000 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.05,.05),  n.mc <- 100000, accept: 13.8%
    % run12 : n.reps <- 5000 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.01,.05),  n.mc <- 100000, accept: 27.8%
    % run18 : like run12 (5000reps) but with set.seed(100) instead of set.seed(3), accept: 24.1%
   
    % run13 : n.reps <- 50 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.01,.05),  n.mc <- 100000, accept: 7.4%
    % run19: like run13 (50reps) but with set.seed(100)  instead of set.seed(3), accept: 3.7%

    % run15 : n.reps <- 100 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.01,.05),  n.mc <- 100000, accept: 10.6%
    % run20: like run15 (100reps) but with set.seed(100)  instead of set.seed(3), accept: 12.7%
    
    % proposal std for sigma: s=small, m=medium, x=large
 
    % nReps = 5000
    loaded = load([loadDir '/run12_MCMC_output.txt']);
    res5000s.theta = exp(loaded); clear loaded
    res5000s.prop = [0.02 0.01 0.05];
    res5000s.start = [3.8 0.3 10];
    res5000s.nReps = 5000;
    res5000s.acceptPerc = 27.8;
  
    loaded = load([loadDir '/run18_MCMC_output.txt']);
    res5000s_b.theta = exp(loaded); clear loaded
    res5000s_b.prop = [0.02 0.01 0.05];
    res5000s_b.start = [3.8 0.3 10];
    res5000s_b.nReps = 5000;
    res5000s_b.acceptPerc = 24.1;
  
    loaded = load([loadDir '/run11_MCMC_output.txt']);
    res5000m.theta = exp(loaded); clear loaded;
    res5000m.prop = [0.02 0.05 0.05];
    res5000m.start = [3.8 0.3 10];
    res5000m.nReps = 5000;
    res5000m.acceptPerc = 13.8;
    
    loaded = load([loadDir '/run10_MCMC_output.txt']);
    res5000x.theta = exp(loaded); clear loaded; 
    res5000x.prop = [0.02 0.1 0.05];
    res5000x.start = [3.8 0.3 10];
    res5000x.nReps = 5000;
    res5000x.acceptPerc = 9;
   
    save([matFolder '/MCMC_nReps5000.mat'],'res5000s','res5000s_b','res5000m','res5000x');
    
    % nReps = 1000
    loaded = load([loadDir '/run9_MCMC_output.txt']);
    res1000s.theta = exp(loaded); clear loaded
    res1000s.prop = [0.02 0.01 0.05];
    res1000s.start = [3.8 0.3 10];
    res1000s.nReps = 1000;
    res1000s.acceptPerc = 27.1;
  
    loaded = load([loadDir '/run22_MCMC_output.txt']);
    res1000s_b.theta = exp(loaded); clear loaded
    res1000s_b.prop = [0.02 0.01 0.05];
    res1000s_b.start = [3.8 0.3 10];
    res1000s_b.nReps = 1000;
    res1000s_b.acceptPerc = 26.0;
    
    loaded = load([loadDir '/run5_MCMC_output.txt']);
    res1000m.theta = exp(loaded); clear loaded;
    res1000m.prop = [0.02 0.05 0.05];
    res1000m.start = [3.8 0.3 10];
    res1000m.nReps = 1000;
    res1000m.acceptPerc = 15.8;
    
    loaded = load([loadDir '/run4_MCMC_output.txt']);
    res1000x.theta = exp(loaded); clear loaded; 
    res1000x.prop = [0.02 0.1 0.05];
    res1000x.start = [3.8 0.3 10];
    res1000x.nReps = 1000;
    res1000x.acceptPerc = 10.7;
   
    save([matFolder '/MCMC_nReps1000.mat'],'res1000s','res1000s_b','res1000m','res1000x');

    % nReps = 500
    loaded = load([loadDir '/run8_MCMC_output.txt']);
    res500s.theta = exp(loaded); clear loaded
    res500s.prop = [0.02 0.01 0.05];
    res500s.start = [3.8 0.3 10];
    res500s.nReps = 500;
    res500s.acceptPerc = 22.1;
    
    loaded = load([loadDir '/run21_MCMC_output.txt']);
    res500s_b.theta = exp(loaded); clear loaded
    res500s_b.prop = [0.02 0.01 0.05];
    res500s_b.start = [3.8 0.3 10];
    res500s_b.nReps = 500;
    res500s_b.acceptPerc = 24.3;
    
    loaded = load([loadDir '/run7_MCMC_output.txt']);
    res500m.theta = exp(loaded); clear loaded;
    res500m.prop = [0.02 0.05 0.05];
    res500m.start = [3.8 0.3 10];
    res500m.nReps = 500;
    res500m.acceptPerc = 12.0;
    
    loaded = load([loadDir '/run6_MCMC_output.txt']);
    res500x.theta = exp(loaded); clear loaded; 
    res500x.prop = [0.02 0.1 0.05];
    res500x.start = [3.8 0.3 10];
    res500x.nReps = 500;
    res500x.acceptPerc = 10.1;
   
    save([matFolder '/MCMC_nReps500.mat'],'res500s','res500s_b','res500m','res500x');

    % nReps = 100
    loaded = load([loadDir '/run15_MCMC_output.txt']);
    res100s.theta = exp(loaded); clear loaded; 
    res100s.prop = [0.02 0.01 0.05];
    res100s.start = [3.8 0.3 10];
    res100s.nReps = 100;
    res100s.acceptPerc = 7.4;

    loaded = load([loadDir '/run20_MCMC_output.txt']);
    res100s_b.theta = exp(loaded); clear loaded; 
    res100s_b.prop = [0.02 0.01 0.05];
    res100s_b.start = [3.8 0.3 10];
    res100s_b.nReps = 100;
    res100s_b.acceptPerc = 12.7;

    save([matFolder '/MCMC_nReps100.mat'],'res100s','res100s_b')

    % nReps = 50
    loaded = load([loadDir '/run13_MCMC_output.txt']);
    res50s.theta = exp(loaded); clear loaded; 
    res50s.prop = [0.02 0.01 0.05];
    res50s.start = [3.8 0.3 10];
    res50s.nReps = 50;
    res50s.acceptPerc = 7.4;

    loaded = load([loadDir '/run19_MCMC_output.txt']);
    res50s_b.theta = exp(loaded); clear loaded; 
    res50s_b.prop = [0.02 0.01 0.05];
    res50s_b.start = [3.8 0.3 10];
    res50s_b.nReps = 50;
    res50s_b.acceptPerc = 3.4;

    save([matFolder '/MCMC_nReps50.mat'],'res50s','res50s_b')

  case 'analyze_MCMC_runs'
    
    
    %% Effect of variance of transition kernel
    % Figure 16
    
    bounds = [[3 5];[0 0.8];[4 20]];
    nBins = 80;
    nGrid = 200;
    xi = zeros(3,nGrid);
    for k=1:3
        xi(k,:)=linspace(bounds(k,1),bounds(k,2),nGrid);
    end
    
    nStart = 25000;
    
    load([matFolder '/MCMC_nReps5000.mat']);
      
    % log r
    para = 1;
    myval = xi(para,:);  
    figNb = figure;
    [hx fx Nx px centerx] = plot_MCMC_res(res5000x.theta(para,:),nStart,myval,nBins,figNb,'b+',[]);
    [hm fm Nm pm centerm] = plot_MCMC_res(res5000m.theta(para,:),nStart,myval,nBins,figNb,'gs',[]);
    [hs fs Ns ps centers] = plot_MCMC_res(res5000s.theta(para,:),nStart,myval,nBins,figNb,'ro',[]);
    set(gca,'ylim',[0 5])
    set(gca,'xlim',[3 4.8])
    legend([hx hm hs],['sigma transition sd: ' num2str(res5000x.prop(2))],...
           ['sigma transition sd: ' num2str(res5000m.prop(2))],...
           ['sigma transition sd: ' num2str(res5000s.prop(2))])
    xlabel('log r')
    ylabel('estimated pdf')
    
    if EXPORT
        fileName = [figFolder '/MCMC_nReps5000_pdf_logr'];
        myexportfig(fileName)
    end
    
    % sigma
    para = 2;
    myval = xi(para,:);
    bw = {[0.01],[0.01],0.02};
    figNb = figure;
    [hx fx Nx px centerx] = plot_MCMC_res(res5000x.theta(para,:),nStart,myval,nBins,figNb,'b+',bw{1});
    [hm fm Nm pm centerm] = plot_MCMC_res(res5000m.theta(para,:),nStart,myval,nBins,figNb,'gs',bw{2});
    [hs fs Ns ps centers] = plot_MCMC_res(res5000s.theta(para,:),nStart,myval,nBins,figNb,'ro',bw{3});
    set(gca,'ylim',[0 7])
    set(gca,'xlim',[-0.01 0.85])  
    legend([hx hm hs],['sigma transition sd: ' num2str(res5000x.prop(2))],...
           ['sigma transition sd: ' num2str(res5000m.prop(2))],...
           ['sigma transition sd: ' num2str(res5000s.prop(2))])
    xlabel('\sigma')
    ylabel('estimated pdf')
   
    if EXPORT
        fileName = [figFolder '/MCMC_nReps5000_pdf_sigma'];
        myexportfig(fileName)
    end
    
    % phi
    para = 3;
    myval = xi(para,:);
    bw = {[],[],[]};
    figNb = figure;
    [hx fx Nx px centerx] = plot_MCMC_res(res5000x.theta(para,:),nStart,myval,nBins,figNb,'b+',bw{1});
    [hm fm Nm pm centerm] = plot_MCMC_res(res5000m.theta(para,:),nStart,myval,nBins,figNb,'gs',bw{2});
    [hs fs Ns ps centers] = plot_MCMC_res(res5000s.theta(para,:),nStart,myval,nBins,figNb,'ro',bw{3});
    set(gca,'ylim',[0 1.2])
    set(gca,'xlim',[6 12])  
    legend([hx hm hs],['sigma transition sd: ' num2str(res5000x.prop(2))],...
           ['sigma transition sd: ' num2str(res5000m.prop(2))],...
           ['sigma transition sd: ' num2str(res5000s.prop(2))],...
           'location','northwest')
    xlabel('\phi')
    ylabel('estimated pdf')
   
    if EXPORT
        fileName = [figFolder '/MCMC_nReps5000_pdf_phi'];
        myexportfig(fileName)
    end
    
    %% Effect of nReps     
    % Figure 17
    
    load([matFolder '/MCMC_nReps5000.mat']);
    load([matFolder '/MCMC_nReps1000.mat']);
    load([matFolder '/MCMC_nReps500.mat']);  
    
    % log r
    para = 1;
    myval = xi(para,:);
   
    figNb=figure;
    [h5000 fs N p center] = plot_MCMC_res(res5000s.theta(para,:),nStart,myval,nBins,figNb,'ro',[]);
    [h1000 fs N p center] = plot_MCMC_res(res1000s.theta(para,:),nStart,myval,nBins,figNb,'gs',[]);
    [h500 fs N p center] = plot_MCMC_res(res500s.theta(para,:),nStart,myval,nBins,figNb,'b+',[]);
    
    set(gca,'ylim',[0 2.5])
    set(gca,'xlim',[3 4.8])
    legend([h5000 h1000 h500],'N = 5000','N = 1000','N = 500')
    xlabel('log r')
    ylabel('estimated pdf')
       
    if EXPORT
        fileName = [figFolder '/MCMC_nRepsEffect_pdf_logr'];
        myexportfig(fileName)
    end
    
    % sigma
    para = 2;
    myval = xi(para,:);
    bw = {0.02,0.05,0.05};

    figNb=figure;
    [h5000 fs N p center] = plot_MCMC_res(res5000s.theta(para,:),nStart,myval,nBins,figNb,'ro',bw{1});
    [h1000 fs N p center] = plot_MCMC_res(res1000s.theta(para,:),nStart,myval,nBins,figNb,'gs',bw{2});
    [h500 fs N p center] = plot_MCMC_res(res500s.theta(para,:),nStart,myval,nBins,figNb,'b+',bw{3});
    
    set(gca,'ylim',[0 7])
    set(gca,'xlim',[-0.01 0.85])  
    legend([h5000 h1000 h500],'N = 5000','N = 1000','N = 500')
    xlabel('\sigma')
    ylabel('estimated pdf')
       
    if EXPORT
        fileName = [figFolder '/MCMC_nRepsEffect_pdf_sigma'];
        myexportfig(fileName)
    end
    
    % phi
    para = 3;
    myval = xi(para,:);
    bw = [];
 
    figNb=figure;
    [h5000 fs N p center] = plot_MCMC_res(res5000s.theta(para,:),nStart,myval,nBins,figNb,'ro',bw);
    [h1000 fs N p center] = plot_MCMC_res(res1000s.theta(para,:),nStart,myval,nBins,figNb,'gs',bw);
    [h500 fs N p center] = plot_MCMC_res(res500s.theta(para,:),nStart,myval,nBins,figNb,'b+',bw);
    
    set(gca,'ylim',[0 0.7])
    set(gca,'xlim',[6 13])  
    legend([h5000 h1000 h500],'N = 5000','N = 1000','N = 500')
    xlabel('\phi')
    ylabel('estimated pdf')
    
    if EXPORT
        fileName = [figFolder '/MCMC_nRepsEffect_pdf_phi'];
        myexportfig(fileName)
    end
   
    %% variability of the results
    % Figure 17
  
    % log r
    para = 1;
    myval = xi(para,:);  

    figNb=figure;
    [h500 fs N p center] = plot_MCMC_res(res500s.theta(para,:),nStart,myval,nBins,figNb,'b+',[]);
    [h500_b fs N p center] = plot_MCMC_res(res500s_b.theta(para,:),nStart,myval,nBins,figNb,'md',[]);  
    
    set(gca,'ylim',[0 2.5])
    set(gca,'xlim',[3 4.8])
    legend([h500 h500_b],{'N = 500',char('N = 500', 'different chain')},'location','northwest')
    xlabel('log r')
    ylabel('estimated pdf') 
    
    if EXPORT
        fileName = [figFolder '/MCMC_nReps500_differentRuns_pdf_logr'];
        myexportfig(fileName)
    end
 
    % sigma
    para = 2;
    myval = xi(para,:);   
  
    figNb=figure;
    [h500 fs N p center] = plot_MCMC_res(res500s.theta(para,:),nStart,myval,nBins,figNb,'b+',0.05);
    [h500_b fs N p center] = plot_MCMC_res(res500s_b.theta(para,:),nStart,myval,nBins,figNb,'md',0.05);
    set(gca,'ylim',[0 7])
    set(gca,'xlim',[-0.01 0.85])  
    legend([h500 h500_b],{'N = 500',char('N = 500', 'different chain')})
    xlabel('\sigma')
    ylabel('estimated pdf')
       
    if EXPORT
        fileName = [figFolder '/MCMC_nReps500_differentRuns_pdf_sigma'];
        myexportfig(fileName)
    end
 
    % phi
    para = 3;
    myval = xi(para,:);
    bw = [];
 
    figNb=figure;
    [h500 fs N p center] = plot_MCMC_res(res500s.theta(para,:),nStart,myval,nBins,figNb,'b+',bw);
    [h500_b fs N p center] = plot_MCMC_res(res500s_b.theta(para,:),nStart,myval,nBins,figNb,'md',bw);
    set(gca,'ylim',[0 0.7])
    set(gca,'xlim',[6 13])  
    legend([h500 h500_b],{'N = 500',char('N = 500', 'different chain')})
    xlabel('\phi')
    ylabel('estimated pdf')
    
    if EXPORT
        fileName = [figFolder '/MCMC_nReps500_differentRuns_pdf_phi'];
        myexportfig(fileName)
    end
  
end
