

## load library
if (is.na(file.info("~/Reference/Rpackages")$isdir)){
 library(sl)
} else {
 library(sl,lib.loc= "~/Reference/Rpackages")}

#  set.seed(3) #all runs till run18
  set.seed(100)  

  n.t <- 50 ## length of data run
  burn.in <- 50 ## burnin length for model
  #n.reps <- 500 ## slow mixing if this lowered 
  n.reps <- 100  

  ## simulate random deviates to re-use for fitting...
  e <- matrix(rnorm((burn.in+n.t)*n.reps),burn.in+n.t,n.reps)
  u <- runif(n.t*n.reps) 

  ## simulate data ....
  #theta <- c(3.8,0.3,10)
  #names(theta) <- c("r","sig.e","phi")
  #Y <- ricker(theta,e,u,burn.in)
  #y <- Y[,1]
  #Yobs <- y

  ## load data
  fileName = '../../matfiles/Ricker/obs_Y.txt';
  Yobs = matrix(scan(fileName),nrow=n.t,ncol=1,byrow="FALSE")


  #n.mc <- 600 ## value for package checking

  n.mc <- 100000 ## length of MCMC run
 
#  theta <- c(4,0.1,6) ## gets dynamics somewhere in right ball park
  theta <- c(3.8,0.3,10) ## Michael
  n.para <- length(theta)
  th <- matrix(0,n.para,n.mc) ## storage for chain results
  llr <- rep(NA,n.mc)         ## storage for log synthetic likelihood
  th[,1] <- log(theta)        ## initial state
 #  prop.sd <- c(.02,.1,.05)    ## proposal standard deviations
  prop.sd <- c(.02,0.01,.05)    ## Michael

  rktrans = NULL
  out = ricker.ll(exp(th[,1]),Yobs,e,u,burn.in=burn.in,trans=rktrans)
  llr[1] <- out$ell
  reject <- 0
  uni <- runif(n.mc)
  pname <- c("r","sig.e","phi")

  for (i in 2:n.mc) { ## MCMC loop
    print(i/n.mc)
    th[,i] <- th[,i-1] + rnorm(n.para)*prop.sd*2
  
    out = ricker.ll(exp(th[,i]),Yobs,e,u,burn.in,trans=rktrans)	
    llr[i] <- out$ell 
  
    alpha <- min(1,exp(llr[i]-llr[i-1]))
  
    if (uni[i]>alpha) { ## reject
      th[,i] <- th[,i-1]
      llr[i] <- llr[i-1]
      reject <- reject + 1
    }
 }

  1-reject/n.mc ## acceptance rate

  start <- round(n.mc/2)

 # par(mfcol=c(1,3))
 # hist(exp(th[1,start:n.mc]))					 
 # hist(exp(th[2,start:n.mc]))					 
 # hist(exp(th[3,start:n.mc]))					 


jobId = 'run20';
fileName = paste(jobId,'MCMC_output.txt', sep="_");
write.table(th, file=fileName, sep = " ", col.names = FALSE, row.names = FALSE, qmethod = "double");


# run1 : n.reps <- 500 theta <- c(3.8,0.3,10),   prop.sd <- c(.02,0.05,.05),  n.mc <- 10000
# run2 : n.reps <- 500 theta <- c(3.8,0.3,10),   prop.sd <- c(.02,0.01,.05),  n.mc <- 10000, accept: 29.75%
# run3 : n.reps <- 500 theta <- c(3.8,0.3,10),   prop.sd <- c(.02,0.01,.05),  n.mc <- 50000, accept: 28.978%
# run3b : n.reps <- 500 theta <- c(3.8,0.3,10),   prop.sd <- c(.02,0.02,.05),  n.mc <- 50000, accept: 17.3%

# run4 : n.reps <- 1000 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.1,.05),  n.mc <- 100000, accept: 10.7%
# run5 : n.reps <- 1000 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.05,.05),  n.mc <- 100000, accept: 15.8%
# run6 : n.reps <- 500 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.1,.05),  n.mc <- 100000, accept : 10.1%
# run7 : n.reps <- 500 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.05,.05),  n.mc <- 100000, accept: 12.0%
# run8 : n.reps <- 500 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.01,.05),  n.mc <- 100000, accept: 22.1%
# run9 : n.reps <- 1000 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.01,.05),  n.mc <- 100000, accept: 27.1 %

# run10 : n.reps <- 5000 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.1,.05),  n.mc <- 100000, accept: 9%
# run11 : n.reps <- 5000 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.05,.05),  n.mc <- 100000, accept: 13.8%
# run12 : n.reps <- 5000 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.01,.05),  n.mc <- 100000, accept: 27.8%

# run13 : n.reps <- 50 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.01,.05),  n.mc <- 100000, accept: 7.4%
# run15 : n.reps <- 100 theta <- c(3.8,0.3,10),  prop.sd <- c(.02,0.01,.05),  n.mc <- 100000, accept: 10.6%


# run18 : like run12 (5000reps) but with set.seed(100) instead of set.seed(3), accept: 24.1%
# run19: like run13 (50reps) but with set.seed(100)  instead of set.seed(3), accept: 3.7%
# run20: like run15 (100reps) but with set.seed(100)  instead of set.seed(3), accept: 12.7%

# run21: like run8 (500reps) but with set.seed(100)  instead of set.seed(3), accept: 24.3 %

# run22: like run9 (1000reps) but with set.seed(100)  instead of set.seed(3), accept: 26.0 %