% --------------------------------------------------
% 
% out = objective(theta,seed,S,Yobs)
%
% theta : K x 3 matrix where each row contains a
%         parameter vector
% seed  : seed to use ([] for current seed)
% S     : number of simulated data sets to use
%         for the computation of the
%         synthetic likelihood (e.g. 500)
% Yobs  : observed data
%
% --------------------------------------------------


function out = objective(theta,seed,S,Yobs)

% generate unique file name for transfer between R and matlab, 
% should also work during parfor.

% maximal file length is 64 (synLogLik.m adds characters to id)
tmp = num2str(floor(rand(1,50)*10));  
tmp = strrep(tmp,' ','');
id = ['./' tmp]; clear tmp;

% Fixed parameters
n = 50; 

%% set seed
if ~isempty(seed)
    currentSeed = rng;
    rng(seed);
end

%% Data generation
burnIn = n;
[out1 out2]= synLogLik(theta,n,Yobs,burnIn,S,id); % minimization

val = -out1; % - ell
logdet = out2;

out{1} = val;

L = length(logdet);
addiInfo = cell(L,1);
for k=1:L
    addiInfo{k} = logdet(k);
end
out{2} = addiInfo;

%% reset seed
if ~isempty(seed)
    rng(currentSeed);
end
  
