% --------------------------------------------------
% 
% yCell = sampleDataDistribution(theta,seed,S)
%
% theta: row vector of parameters, e.g. theta = [3.8 0.3 10];
% seed : seed to be used. If empty, current seed is
%         used
% S    : number of data sets to sample, e.g. S=25
%
% yCell: 1xS cell array of S data sets
%
% --------------------------------------------------


function yCell = sampleDataDistribution(theta1,seed,S)

% generate unique file name for transfer between R and matlab, 
% should also work during parfor.

% maximal file length is 64 (simulateRicker adds characters to id)
tmp = num2str(floor(rand(1,50)*10));  
tmp = strrep(tmp,' ','');
id = ['./' tmp]; clear tmp;

% Fixed paramters
n = 50; 
theta2Fixed = 0.3;
theta3Fixed = 10;

theta = [theta1 theta2Fixed theta3Fixed];

%% set seed
if ~isempty(seed)
    currentSeed = rng;
    rng(seed);
end

%% Data generation
burnIn = n;
yCell = simulateRicker(theta,n,burnIn,S,id);

%% reset seed
if ~isempty(seed)
    rng(currentSeed);
end
  
