% --------------------------------------------------
%
% objfun: function handler pointing to a function taking theta as input and returning the distance
% prior.rnd : function handler for random number generation. Take sample size as input and returns the samples (dim x number of samples). Omit if
% options.loadFile is specified.
% prior.pdf: function handler to the pdf of the prior (takes dim x 1 vector as input and returns a scalar)
% T : number of iterations in the sequential ABC
% options.quant_start : starting quantile to determine epsilon of the first iteration (default: 0.75). Not needed if options.loadFile is specified
% options.epsi_quant : quantile for choosing next epsilon (default: 0.1)
% options.fileName : where to save the results after each iteration (default:  ['tmp_result_SABC ' date]);
% options.loadFile : possible file where an initial values of 'theta','weights','myDist','epsi' are stored. If not specified, the first iteration
% is rejection sampling.
% options.dim : dimensionality of the parameter
% options.sampleSize : number of samples to generate
% options.epsiMin : the smallest allowed threshold value (default: 0)
%
% Implements algorithm 4 "Likelihood-free population Monte Carlo sampler" of the article
% Marin, J.-M.; Pudlo, P.; Robert, C. & Ryder, R. 
% Approximate Bayesian computational methods Statistics and Computing, 
% Springer Netherlands, 2011, 1-14 
%
% --------------------------------------------------

function cluster_SABC_init(objfun,prior,options)

fileName = options.fileName;
N = options.sampleSize;
p = options.dim;

T = 1;
allEpsi=cell(T,1);
allTheta=cell(T,1);
allWeights=cell(T,1);
allDist = cell(T,1);
allCost = cell(T,1);

t=1;
theta = zeros(p,N);
myDist = zeros(1,N);
    
thetap = zeros(p,N);
myDist = zeros(1,N);
parfor ind=1:N
    fprintf(['\n' num2str(ind/N) '\n'])
    
    theta(:,ind) = prior.rnd(1);
    myDist(1,ind) = objfun(theta_prop(:,ind));
end
            
weights = ones(1,N);
        
% Save
allTheta{t} = theta;
allWeights{t} = weights;
allDist{t} = myDist;
allEpsi{t} = Inf;
allCost{t} = nSimulations;

save(fileName,'allTheta','allWeights','allDist','allEpsi','allCost','t','options');


