% --------------------------------------------------
%
% cluster_SABC_run(nb)
%
% Runs SABC on a cluster. The target generation
% needs to be set by hand (nGeneration)
%
% --------------------------------------------------

function cluster_SABC_run(nb)

% set the target generation
% (if not zero, assumes that the results for nGeneration-1 were already computed)
nGeneration = 0;

startup;

matFolder = '../../matfiles/bacterial_infections';

% fixed parameters
load([matFolder '/binaryRealData.mat'],'sample_sizes','global_strain_distribution','data_epi')

% sampled individuals in each day care center (DCC)
fixedParams.DCC_sample_sizes = sample_sizes; clear sample_sizes;

% Overall distribution of the different strains in the data
fixedParams.global_strain_distribution = global_strain_distribution/sum(global_strain_distribution); clear global_strain_distribution;

% average number of subjects in the community/group
fixedParams.avg_DCC_size = 53; 
fixedParams.simulation_time = 10;


sumStatsObs =  expertSummaryStatistics(data_epi,fixedParams);
N = 1;

LOG_TRANSFORM = 0;
if LOG_TRANSFORM
    transf = @(theta)transform2norm(theta,[3,3,3],[10.99,1.99,0.99],[0.01,0.01,0.01]);
    invtransf = @(theta)transforminv(theta,[3,3,3],[10.99,1.99,0.99],[0.01,0.01,0.01]);
    prior.param = {[0.01,10.99],[0.01,1.99],[0.01,0.99]}; 
    prior.type = [{'Uniform'},{'Uniform'},{'Uniform'}];
    prior.rnd=@(n)samplePrior(n,prior.type,prior.param,transf);    
    prior.pdf=@(theta)priorPdf_with_transform(theta,prior.type,prior.param,invtransf);
    obj = @(theta)(SABC_objective(invtransf(theta'),N,sumStatsObs,fixedParams));
else
    bounds = [ [0 11];[0 2];[0 1] ];
    obj = @(theta)(SABC_objective(theta',N,sumStatsObs,fixedParams));
    prior.pdf = @(theta)(priorPdf(theta,bounds));
    prior.rnd = @(n)(priorRnd(n,bounds));
end

options.fileName = [matFolder '/SABC_gen' num2str(nGeneration) '_res' num2str(nb) '.mat'];
options.dim = 3;
options.sampleSize = 50;

rng('shuffle')

if nGeneration == 0
    cluster_SABC_init(obj,prior,options);
elseif nGeneration > 0
    loaded = load([matFolder '/SABC_gen' num2str(nGeneration-1) '.mat'],'theta','weights','threshold');
    prevTheta = loaded.theta;
    prevWeights = loaded.weights;
    epsi = loaded.threshold;
    
    cluster_SABC_cont(obj,prior,prevTheta,prevWeights,options,epsi);
end
    
