
% --------------------------------------------------
%
%  [acq, grad] = acqCost(evidence,options,u)
%  acq = acqCost(evidence,options,u)
%
%  computes the acquisition cost at the
%  row vectors u(k,:), k=1...m
%
%  acq  : m x 1
%  grad : m x size(u,2)
%
% --------------------------------------------------


function [acq, grad] = acqCost(evidence,options,u)

if isfield(options.ACQ,'acqLogPrior') && ~isempty(options.ACQ.acqLogPrior)
    [acqLogPrior acqLogPrior_grad]= options.ACQ.acqLogPrior(u);
else
    acqLogPrior = 0;
    acqLogPrior_grad = 0;
end
    
if strcmp(options.ACQ.type,'LCB')
    % UCB for maximization
    nu = 1;
    coeff = sqrt(nu*2*log(evidence.nX^(options.GP.dim/2+2)*pi^2 /(3*0.1)));
       
    if nargout == 1
        [postMean postVar_latent postVar] = post_mean_var(evidence,options.GP,u);
        postSigma_latent =  sqrt(postVar_latent);
        acq = -acqLogPrior+postMean-coeff*postSigma_latent; % we consider minimization
               
    elseif nargout == 2
        %fprintf('acqCost gradient \n')
        [postMean postVar_latent postVar postMean_grad postVar_grad] = post_mean_var(evidence,options.GP,u);
     
        postSigma_latent =  sqrt(postVar_latent);
        acq = -acqLogPrior+postMean-coeff*postSigma_latent;
        grad = -acqLogPrior_grad+postMean_grad - coeff*1./(2*postSigma_latent).*postVar_grad;

    end
        
elseif strcmp(options.ACQ.type,'EI')
    [postMean postVar_latent postVar] = post_mean_var(evidence,options.GP,u);
    
    postMean = postMean-acqLogPrior;
    postSigma_latent =  sqrt(postVar_latent);
    acq = -log_exp_imp(evidence.best_val, (postMean-0.0001), postSigma_latent);
else
    error('Acquisition function not implemented')  
end

