% --------------------------------------------------
%
%  [fxacq xacq] = acquire(evidence,options,obj) 
% 
%  Acquires a new point (xacq,fxacq) using the 
%  provided evidence, fxacq = obj(xacq)
%
% xacq : row vector
%
% --------------------------------------------------

function [fxacq xacq xBest myBox] = acquire(evidence,options,obj,nPoints) 

fprintf('Optimize acquisition function ... ')
ticStart = tic;
bounds = options.ACQ.optim.bounds;
initGuess = unifrnd(bounds(:,1),bounds(:,2));

f = @(u)acqCost_colVector(evidence,options,u);
xBest = bndminimize(initGuess,f,bounds,options.ACQ.optim)';
time = toc(ticStart);
fprintf([' (' num2str(time) ' s) \n'])

if options.ACQ.acqSlack == 0
    xacq = repmat(xBest,nPoints,1);
    myBox = [xBest' xBest'];
else
    myBox = findBox(evidence,options,xBest);    
    boxLength = myBox(:,2)-myBox(:,1);
    sd = boxLength/2;
    xacq = zeros(nPoints,options.GP.dim);
    for k=1:nPoints
        noise = bsxfun(@times,sd,randn(options.GP.dim,1));
        tmp = bsxfun(@plus,xBest',noise);
        if sum(tmp>options.ACQ.optim.bounds(:,2)) | sum(tmp<options.ACQ.optim.bounds(:,1))
            tmp = rand(options.GP.dim,1).*boxLength+myBox(:,1);
        end
        xacq(k,:)=tmp'; 
        clear tmp;
    end
end
    

fprintf('Evaluate objective ... ')
ticStart = tic;
fxacq = obj(xacq);

time = toc(ticStart);
fprintf([' (' num2str(time) ' s) \n'])

