% --------------------------------------------------
%
% SMC(likfun,priorPdf,T,iniPoints,options)
%
% likfun: function handler pointing to a function taking theta as input and returning the likelihood
% priorPdf: function handler to the pdf of the prior (takes dim x 1 vector as input and returns a scalar)
% T : number of iterations in the sequential algoritm ("generations")
% iniPoints: initial points from where the sampling starts. Leave empty
% if options.loadName is specified.
% options.fileName : where to save the results after each iteration (default:  ['tmp_result_SABC ' date]);
% options.loadFile : possible file where an initial values of 'theta','weights','myLik' are stored. If not specified, the first iteration
% uses the iniPoints as starting points
% options.dim : dimensionality of the parameter
% options.sampleSize : number of samples to generate
%
%
% --------------------------------------------------

function SMC(likfun,priorPdf,T,iniPoints,options,covFactor)

%% Default options

if ~isfield(options,'fileName')
    fileName = ['tmp_result_SMC ' date];
else
    fileName = options.fileName;
end

N = options.sampleSize;
p = options.dim;

allTheta=cell(T,1);
allWeights=cell(T,1);
allLiks = cell(T,1);


%%Initial Sampling Step t=1


if ~isfield(options,'loadFile')

    t = 1;
    
    Sigma = covFactor*cov(iniPoints');
    
    theta = zeros(p,N);
    myLik = zeros(1,N);
    weights = zeros(1,N);
    
    nIni = size(iniPoints,2);
    allWeights0 = 1/nIni*ones(1,nIni);
    
    parfor ind=1:N        
        fprintf(['\n' num2str(ind/N) '\n'])
        
        index = sum(mnrnd(1,allWeights0).*[1:nIni]);
        theta(:,ind) = mvnrnd(iniPoints(:,index)',Sigma,1)';      
        myLik(1,ind) = likfun(theta(:,ind));
            
        likp = myLik(1,ind)*priorPdf(theta(:,ind));
        if likp == 0
            weights(ind) = 0;
        else
            weights(ind) = likp/( allWeights0*mvnpdf(theta(:,ind)',iniPoints',Sigma) );
        end
    end
    
    if sum(weights)>0
        allWeights{t}=weights/sum(weights);
    else
        allWeights{t}=weights;
    end
 
    allTheta{t}=theta;
    allLiks{t} = myLik;

    save(fileName,'allTheta','allWeights','allLiks','t','options');
    
    tStart = 2;
else
    loaded = load(options.loadFile,'allTheta','allWeights','allLiks','t');
    for kt = 1:loaded.t
        allTheta{kt} = loaded.allTheta{kt};
        allWeights{kt} = loaded.allWeights{kt};
        allLiks{kt} = loaded.allLiks{kt};
    end
        tStart = loaded.t+1; clear loaded;
end
    
%% Remaining iterations

for t=tStart:T
       
    [Sigma dummy] = weighted_cov(allTheta{t-1},allWeights{t-1});
    Sigma = covFactor*Sigma;
    
    theta = zeros(p,N);
    myLik = zeros(1,N);
    weights = zeros(1,N);
    
    parfor ind=1:N
            
        fprintf(['\n' num2str(ind/N) '\n'])
        
        index = sum(mnrnd(1,allWeights{t-1}).*[1:N]);
        theta(:,ind) = mvnrnd(allTheta{t-1}(:,index)',Sigma,1)';      
        myLik(1,ind) = likfun(theta(:,ind));
            
        likp = myLik(1,ind)*priorPdf(theta(:,ind));
        if likp == 0
            weights(ind) = 0;
        else
            weights(ind) = likp/( allWeights{t-1}*mvnpdf(theta(:,ind)',allTheta{t-1}',Sigma) );
        end
        
    end
    if sum(weights)>0
        allWeights{t}=weights/sum(weights);
    else
        allWeights{t}=weights;
    end
    allTheta{t}=theta;
    allLiks{t} = myLik;

    save(fileName,'allTheta','allWeights','allLiks','t','options');
end

