% --------------------------------------------------
%
% [logLik logLik_grad] = logLikApprox(evidence,options,x,bandwidth,method)
%
%
% Converts the distribution of the distance to the log-likelihood
%
% bandwidth : if empty 0.1 (for uniform kernel) or 0.2 (for Gaussian kernel)
% quantiles are taken. For synlik, default is h=1;
% needed if method is 'GP-synlik' or 'logGP-synlik'
%
% method : 'GP-uniform'
%          ('logGP-uniform')
%          ('GP-synlik')
%          'logGP-synlik'
%          ('GP-Gaussian')
%          ('logGP-Gaussian')
%
% () means not thoroughly tested!
%
% x      : K parameters as rows 
% logLik : K x 1 
% logLik_grad : K x d (where d is size(x,2))
%
% --------------------------------------------------


function [logLik logLik_grad] = logLikApprox(evidence,options,x,bandwidth,method)

thetaHat = getThetaJ(evidence);

bwQuantile = 0.05;

switch method
    
    case 'GP-uniform'
      if isempty(bandwidth)
          [postMeanHat dummy postVarHat] = post_mean_var(evidence,options.GP,thetaHat);
          postSigmaHat = sqrt(postVarHat*options.OBJ.N);
          bandwidth = norminv(bwQuantile,postMeanHat,postSigmaHat);
      end
      if nargout == 1
          lik = likApproxUniformKernelGP(evidence,options,x,bandwidth);
          logLik = log(lik);
      elseif nargout == 2
          [lik lik_grad] = likApproxUniformKernelGP(evidence,options,x,bandwidth);
          logLik = log(lik);
          logLik_grad = bsxfun(@times,lik_grad,lik);
      end
      
  case 'logGP-uniform'
    if isempty(bandwidth)
        [postMeanHat dummy postVarHat] = post_mean_var(evidence,options.GP,thetaHat);
        postSigmaHat = sqrt(postVarHat*options.OBJ.N);
        bandwidth = exp(postMean+postSigmaHat*norminv(bwQuantile));
    end
    if nargout == 1
        lik = likApproxUniformKernelGP(evidence,options,x,bandwidth);
        logLik = log(lik);
    elseif nargout == 2
        [lik lik_grad] = likApproxUniformKernelGP(evidence,options,x,bandwidth);
        logLik = log(lik);
        logLik_grad = bsxfun(@times,lik_grad,lik);
    end
    
  case 'GP-synlik'
    if isempty(bandwidth)
        bandwidth = 1;
    end
    if nargout == 1
        postMean =  post_mean(evidence,options.GP,x);
        Jhat = postMean;
        logLik = -1/(2*bandwidth^2)*Jhat;
    elseif nargout == 2
        [postMean postMean_grad] =  post_mean(evidence,options.GP,x);
        Jhat = postMean;
        logLik = -1/(2*bandwidth^2)*Jhat;
        logLik_grad = -1/(2*bandwidth^2)*postMean_grad;
    end
    
  case 'logGP-synlik'
       if isempty(bandwidth)
           bandwidth = 1;
       end
    if nargout == 1
        [postMean postVar_latent postVar] = post_mean_var(evidence,options.GP,x);
        Jhat = exp(postMean+0.5*postVar);
        logLik = -1/(2*bandwidth^2)*Jhat;
    elseif nargout == 2
        [postMean postVar_latent postVar postMean_grad postVar_grad]= post_mean_var(evidence,options.GP,x);    
        Jhat = exp(postMean+0.5*postVar);
        logLik = -1/(2*bandwidth^2)*Jhat;
        logLik_grad = -1/(2*bandwidth^2)*Jhat.*(postMean_grad+0.5*postVar_grad);
    end

  case 'GP-Gaussian'
    % mode instead of mean: exp(-x) = exp(-mu)
    if isempty(bandwidth)
        [postMeanHat dummy postVarHat] = post_mean_var(evidence,options.GP,thetaHat);
        postSigmaHat = sqrt(postVarHat*options.OBJ.N);
        bandwidth = norminv(bwQuantile,postMeanHat,postSigmaHat)/4;
    end
   
    if nargout == 1
        postMean =  post_mean(evidence,options.GP,x);
        Jhat = postMean;
        logLik = -1/(2*bandwidth^2)*Jhat;
    elseif nargout == 2
        [postMean postMean_grad] =  post_mean(evidence,options.GP,x);
        Jhat = postMean;
        logLik = -1/(2*bandwidth^2)*Jhat;
        logLik_grad = -1/(2*bandwidth^2)*postMean_grad;
    end

  case 'logGP-Gaussian'
    % mode instead of mean
    if isempty(bandwidth)
        [postMeanHat dummy postVarHat] = post_mean_var(evidence,options.GP,thetaHat);
        postSigmaHat = sqrt(postVarHat*options.OBJ.N);
        bandwidth = exp(postMean+postSigmaHat*norminv(bwQuantile))/4;
    end
    if nargout == 1
        [postMean postVar_latent postVar] = post_mean_var(evidence,options.GP,x);
        Jhat = exp(postMean+0.5*postVar);
        logLik = -1/(2*bandwidth^2)*Jhat;
    elseif nargout == 2
        [postMean postVar_latent postVar postMean_grad postVar_grad]= post_mean_var(evidence,options.GP,x);    
        Jhat = exp(postMean+0.5*postVar);
        logLik = -1/(2*bandwidth^2)*Jhat;
        logLik_grad = -1/(2*bandwidth^2)*Jhat.*(postMean_grad+0.5*postVar_grad);
    end
    
end



