
% --------------------------------------------------
%
% out = minimize_post_mean(evidence,options,initGuess)
%
% Minimizes the regression function
% initGuess: row vector, can be []
% out : row vector with minimizer
%
% --------------------------------------------------

function out = minimize_regression_function(evidence,options,initGuess)
       
    if isempty(initGuess)
        initGuess = evidence.best_x; % row vector
    end
           
    f = @(x)thetaHatCost(evidence,options,x);
    bounds = options.ACQ.optim.bounds;
    optimoptions = options.ESTIM.optim;
    
    fprintf('Optimize posterior mean function ... ')
    ticStart = tic;
    out = bndminimize(initGuess',f,bounds,optimoptions)'; %bndminimize expects column vectors
    time = toc(ticStart);
    fprintf([' (' num2str(time) ' s) \n'])

    

function [val grad] = thetaHatCost(evidence,options,x)
% x: column vectors
% grad : column vector

    if nargout == 2
       
        if options.GP.log == 1 % log-GP
            [postMean postVar_latent postVar postMean_grad postVar_grad]=post_mean_var(evidence,options.GP,x');
            val = exp(postMean+0.5*postVar);
            grad = bsxfun(@times,postMean,(postMean_grad+0.5*postVar_grad));
        else
            [val grad] = post_mean(evidence,options.GP,x');
        end
            
        if isfield(options.ACQ,'acqLogPrior') && ~isempty(options.ACQ.acqLogPrior)
            [acqLogPrior acqLogPrior_grad]= options.ACQ.acqLogPrior(x);
            
            val = val-acqLogPrior;
            grad = grad-acqLogPrior_grad;
        end
        
        grad = grad';
    
    elseif nargout == 1
        
        if options.GP.log == 1 % log-GP
            [postMean postVar_latent postVar] =post_mean_var(evidence,options.GP,x');
            val =  exp(postMean+0.5*postVar);
        else
            val = post_mean(evidence,options.GP,x');
        end
        
        if isfield(options.ACQ,'acqLogPrior') && ~isempty(options.ACQ.acqLogPrior)
            acqLogPrior = options.ACQ.acqLogPrior(x);
            val = val-acqLogPrior;
        end
    
    end
