% samplingOptions.nSamples : number of samples
% samplingOptions.method : 'SMC'
% samplingOptions.priorPdf: takes as input a vector of parameters and
%                            returns values of prior pdf (SMC), or log pdf (HMC)
% samplingOptions.nGenerations : (for SMC) the number of generations
% samplingOptions.fileName: where to save the results

%
% likOptions.bandwidth : [], or h>0
% likOptions.method    :  see logLikApprox.m

function sample_posterior(evidence,options,likOptions,samplingOptions)

bandwidth = likOptions.bandwidth;
likMethod = likOptions.method;
samplingMethod = samplingOptions.method;


switch samplingMethod
  case 'SMC'
    SMC_options.fileName = samplingOptions.fileName;
    SMC_options.sampleSize = samplingOptions.nSamples;
    SMC_options.dim = options.GP.dim;
    nGenerations = samplingOptions.nGenerations;
    iniPoints = evidence.X(1:evidence.nX,:)';
    priorPdf = samplingOptions.priorPdf;

    if isfield(samplingOptions,'transf') && ~isempty(samplingOptions.transf)
        iniPoints = samplingOptions.transf(evidence.X(1:evidence.nX,:))';
        invtransf = samplingOptions.invtransf;
        likfun = @(theta)(likfun1(invtransf(theta'),evidence,options,bandwidth,likMethod));
        covFactor = 1;    
    else
        likfun = @(theta)(likfun1(theta',evidence,options,bandwidth,likMethod));
        covFactor = 1/3;    
    end


    SMC(likfun,priorPdf,nGenerations,iniPoints,SMC_options,covFactor);
    
  case 'modelSABC'
    
    
  otherwise
    error('sample_posterior.m : sampling method is not implemented');
    
end

function lik = likfun1(x,evidence,options,bandwidth,method)
% x: row vector

logLik = logLikApprox(evidence,options,x,bandwidth,method);
lik = exp(logLik);


