% --------------------------------------------------
%
%  evidence = L_alpha_priormean(evidence,optionsGP)
%
% Computes L in LL'=Ky, alpha = Ky^{-1} (f-priormean), and 
% prior mean, and stores the results in the evidence 
% struct.
%
% alpha is computed via the Cholesky decomposition
%
% --------------------------------------------------

function evidence = L_alpha_priormean(evidence,optionsGP)

x = evidence.X(1:evidence.nX,:);
y = evidence.f(1:evidence.nX);

hyp = evidence.hyp;

m = prior_mean(hyp.mean,optionsGP.mean,x);
noiseVar = exp(2*hyp.lik);
K = feval(optionsGP.cov{:},hyp.cov,x,x);
Ky = K+eye(evidence.nX)*noiseVar;
L = sparse(chol(Ky, 'lower')); % L L' = Ky
df = y-m;
alpha = solve_chol(full(L)',df); %solves Ky alpha = df, i.e. computes Ky^{-1} df 

evidence.L = L;
evidence.alpha = alpha;
evidence.prior_mean(1:evidence.nX) = m;

