% --------------------------------------------------
%
% evidence = fit_mean_function(evidence,optionsGP,mode)
%
% Fits the prior mean function using the covariance matrix
% K = evidence.L*evidence.L' if mode=='L', else if 
% mode == 'identity', the identity is used
%
% --------------------------------------------------

function evidence = fit_mean_function(evidence,optionsGP,mode)

    nX = evidence.nX;
    Y = evidence.f(1:nX);
    
    if strcmp(char(optionsGP.mean{:}),'meanQuadratic')
        x = evidence.X(1:nX,:); 
        X = [x.^2 x ones(nX,1)];
    elseif strcmp(char(optionsGP.mean{:}),'meanConst')
        X = ones(nX,1);
    else
        error(['Mean function ' char(optionsGP.mean{:}) ' not supported'])
    end
    
    if strcmp(mode,'identity')
        KinvY = Y;
        KinvX = X;
    elseif strcmp(mode,'L')
        tmp = L_alpha_priormean(evidence,optionsGP);
        L = tmp.L;
        KinvY = solve_chol(full(L)',Y);
        KinvX = solve_chol(full(L)',X);
    end
    
    XKY = X'*KinvY;
    XKX = X'*KinvX;
    % XKX should be symmetric, this corrects for numerical deviations
    XKX = 0.5*(XKX+XKX'); 
      
    if optionsGP.optim.noBounds
        evidence.hyp.mean = (XKX)\(XKY);
    else
        q_ini = evidence.hyp.mean;
      
        lb = optionsGP.optim.Lbounds.mean;
        ub = optionsGP.optim.Ubounds.mean;
        
        bounds = [lb ub];
        optimfun = @(in)(cost(in,XKX,XKY));
        opt.useGradient = 1;
        opt.TMP.method = 'lbfgs'; % 'sd', 'lbfgs'
        opt.TMP.maxIter = 300;
        opt.TMP.optTol = 1e-8;
        q_bnd = bndminimize(q_ini,optimfun,bounds,opt);

        opts = optimset('Display','off');
        q_matlab = quadprog(XKX,-XKY',[],[],[],[],lb,ub,[],opts);

        val_ini = optimfun(q_ini);
        val_matlab = optimfun(q_matlab);
        val_bnd = optimfun(q_bnd);
        
        if val_matlab < val_bnd
            q = q_matlab;
            best_val = val_matlab;
        else
            q = q_bnd;
            best_val = val_bnd;
        end
    
        if best_val<val_ini
            evidence.hyp.mean = q;
        end

    end

   
    
function [val grad] = cost(q,XKX,XKY)

    val = -q'*XKY+0.5*q'*XKX*q;
    if nargout == 1
        return;
    elseif nargout == 2
        grad = -XKY+XKX*q;
    end