function [A grad] = meanQuadratic(hyp, x, i)

% The mean function is parameterized as:
%
% m(x) = sum_i a_i * x_i.^2 + sum_i b_i * x_i +c ;
%
% The hyperparameter is:
%
% hyp = [ a_1
%         a_2
%         ..
%         a_D
%         b_1
%         b_2
%         ..
%         b_D
%         c ]
%
% Added by Michael Gutmann
%

if nargin<2, A = '2*D+1'; return; end             % report number of hyperparameters 
[n,D] = size(x);
if any(size(hyp)~=[2*D+1,1]), error('Exactly 2D+1 hyperparameters needed.'), end

xx = [x.^2 x ones(n,1)];

if nargin==2
    % evaluate mean
    A = xx*hyp;
else
    % derivative with respect to hyperparameter i
  if i<=2*D+1
    A = xx(:,i);                               
  else
    A = zeros(n,1);
  end
end

if nargout==2
     grad = 2*bsxfun(@times,hyp(1:D)',x);
     grad = bsxfun(@plus,grad,hyp(D+1:2*D)');
end