% --------------------------------------------------
%
%  evidence = learn_hyper_param(evidence,optionsGP,mode)
%
%  mode : 'init','middle','end'
%
% --------------------------------------------------


function evidence = learn_hyper_param(evidence,optionsGP,mode)

    switch mode
      case 'init'
        myopt = optionsGP.optim.init;
      case 'middle'
        myopt = optionsGP.optim.middle;
      case 'end';
        myopt = optionsGP.optim.end;
        if  evidence.nX < myopt.reqDataPoints
            return;
        end
    end  
    
    fprintf('Learn hyper parameters ... ')
    ticStart = tic;
    %% Update hyperparameters
    
    % Fit parameters of the mean function only
    if strcmp(mode,'init')
        evidence = fit_mean_function(evidence,optionsGP,'identity');
    end
    
    % Fit all parameters
    optimfun = @(in)(fct(in,evidence,optionsGP)); 
    
    if optionsGP.optim.noBounds
        fprintf('\n')
        hyp = minimize(unwrap(evidence.hyp),optimfun,myopt.maxIter);
    else   
        lowerBounds = unwrap(optionsGP.optim.Lbounds);
        upperBounds = unwrap(optionsGP.optim.Ubounds);
        bounds = [lowerBounds upperBounds]; 
        hyp = bndminimize(unwrap(evidence.hyp),optimfun,bounds,myopt);
    end
    evidence.hyp = rewrap(evidence.hyp,hyp);
  
   
    
    %% Recompute kernel and prior mean
    evidence = L_alpha_priormean(evidence,optionsGP);
    time = toc(ticStart);
    fprintf([' (' num2str(time) ' s) \n'])
 
% --------------------------------------------------
    
    
function [val grad] = fct(in,evidence,optionsGP)
    
    x = evidence.X(1:evidence.nX,:);
    y = evidence.f(1:evidence.nX);
    
    myHyp = rewrap(evidence.hyp,in);
    infMethod = optionsGP.optim.infMethod;
    
    [val grad] = gp(myHyp,infMethod,optionsGP.mean,optionsGP.cov,optionsGP.lik, x, y);
    grad = unwrap(grad);
    
    