% --------------------------------------------------
%
%  evidence = update_L_alpha_priormean(evidence,optionsGP,k_x,k_xx)
%
% Updates the Cholesky decomposition LL'=Ky, and the prior mean,
% and re-computes alpha = Ky^{-1} (f-priormean), and stores 
% the results in the evidence struct.
%
% alpha is computed via the Cholesky decomposition
%
% --------------------------------------------------

function evidence = update_L_alpha_priormean(evidence,optionsGP,k_x,k_xx)

x = evidence.X(1:evidence.nX,:);
y = evidence.f(1:evidence.nX);
xnew = x(end,:); % last entry is the new one

hyp = evidence.hyp;

mnew = prior_mean(hyp.mean,optionsGP.mean,xnew);
m = [evidence.prior_mean(1:evidence.nX-1);mnew];
noiseVar = exp(2*hyp.lik);
z_t = evidence.L\k_x; % L^-1 k_x
d_t = sqrt(k_xx + noiseVar - z_t'*z_t);
L = sparse([[full(evidence.L), zeros(evidence.nX-1, 1)];[z_t', d_t]]);
df = y-m;
alpha = solve_chol(full(L)',df); %solves Ky alpha = df, i.e. computes Ky^{-1} df

evidence.L = L;
evidence.alpha = alpha;
evidence.prior_mean(evidence.nX) = mnew;

