% --------------------------------------------------
%
% h=quickPlot(evidence,options,LOGSCALE,varargin)
% h=quickPlot(evidence,options,LOGSCALE)
% h=quickPlot(evidence,options,LOGSCALE,dim)
% h=quickPlot(evidence,options,LOGSCALE,dim,range,SHOW_PRIOR_MEAN,SHOW_ACQ)
%
% plot the posterior mean and quantiles along dimension 
% dim in range. LOGSCALE \in {0,1}. If LOGSCALE = 1, 
% the plot is in the log-domain. 
%
% --------------------------------------------------


function h=quickPlot(evidence,options,LOGSCALE,varargin)

    quants = [0.1 0.9];

    if nargin == 3
        dim = 1;
        SHOW_PRIOR_MEAN = 0;
        SHOW_ACQ = 0;
        range = [];
    elseif nargin == 4
        dim = varargin{1};
        SHOW_PRIOR_MEAN = 0;
        SHOW_ACQ = 0;
        range = [];
    else
        dim = varargin{1};
        range = varargin{2};
        SHOW_PRIOR_MEAN = varargin{3};
        SHOW_ACQ = varargin{4};
    end         
    
    if isempty(range)
        range = options.ACQ.optim.bounds(dim,:);
    end
          
     
    if isfield(options.ACQ,'acqLogPrior') && ~isempty(options.ACQ.acqLogPrior)
        acqLogPrior = options.ACQ.acqLogPrior;
    else
        acqLogPrior = @(u)(0);
    end
       
    acqLogPrior = @(u)(0);
            
    %% gathered data
    nX = evidence.nX;
    x = evidence.X(1:nX,dim);
    y = evidence.f(1:nX);
    if options.GP.log == 1
        y = exp(y);
    end
    y = y - acqLogPrior(x);
    
    if LOGSCALE 
        y = log(y);
    end
    
    counter = 1;
    h(counter)=plot(x,y,'o');
    xlabel(['theta' num2str(dim)])
    if LOGSCALE 
        ylabel('log objective')
    else
        ylabel('objective')
    end
    counter = counter+1;
    hold on
    
    %% estimates
    thetaHat = getThetaJ(evidence);

    u = repmat(thetaHat,100,1);
    uu = linspace(range(1),range(2),100)';
    u(:,dim) = uu;
             
    %% prior mean
    if SHOW_PRIOR_MEAN
        priorMean = prior_mean(evidence.hyp.mean,options.GP.mean,u)-acqLogPrior(u);
        
        if LOGSCALE
            priorMean = log(priorMean);
        end
        
        h(counter) = plot(uu,priorMean,'c');
        counter = counter + 1;
    end
   
    %% posterior mean and quantiles
    [postMean postVar_latent postVar] = post_mean_var(evidence,options.GP,u);
    if options.GP.log == 1
        postJmean =  exp(postMean+0.5*postVar);
    else
        postJmean = postMean;
    end
    postJmean = postJmean-acqLogPrior(u);
     
    nU = size(u,1);
    postJquants = zeros(nU,length(quants));
        
    for k=1:length(quants)
        if options.GP.log == 1
            postJquants(:,k) = exp(postMean+sqrt(postVar)*norminv(quants(k)))-acqLogPrior(u);
        else
            postJquants(:,k) = norminv(quants(k),postMean,sqrt(postVar))-acqLogPrior(u);
        end
    end
        
    if LOGSCALE
        postJmean = log(postJmean);
        postJquants = log(postJquants);
    end
      
    h(counter)=plot(uu,postJmean,'k'); counter = counter + 1;
    h(counter)=plot(uu,postJquants(:,1),'k--'); counter = counter + 1;
    plot(uu,postJquants(:,2),'k--');

    %% acquisition function    
    if  SHOW_ACQ
        acquisitionCost = acqCost(evidence,options,u);
        
        h(counter)=plot(uu,acquisitionCost,'g'); 
    end
    
        
    %% plot
    if SHOW_PRIOR_MEAN && SHOW_ACQ
        legend(h,'data','prior mean','post mean',[num2str(quants(1)) ',' num2str(quants(2)) ' quantiles'],'acquisition function','location','best')
    elseif SHOW_PRIOR_MEAN && ~SHOW_ACQ
        legend(h,'data','prior mean','post mean',[num2str(quants(1)) ',' num2str(quants(2)) ' quantiles'],'location','best')
    elseif ~SHOW_PRIOR_MEAN && SHOW_ACQ
        legend(h,'data','post mean',[num2str(quants(1)) ',' num2str(quants(2)) ' quantiles'],'acquisition function','location','best')
    end
        
    title(['Data points: ' num2str(evidence.nX) ' thetaHat' num2str(dim) ' = ' num2str(thetaHat(dim))])
        
    set(gca,'xlim',range)
    