% --------------------------------------------------
%
% h=quickPlot2D(evidence,options,dims,LOGSCALE,varargin)
% LOGSCALE = 1: plots in log scale
% varargin{1}: number of contours
%
% --------------------------------------------------


function h=quickPlot2D(evidence,options,dims,LOGSCALE,varargin)

    WITH_SCATTER = 1;
    
    dim1 = dims(1);
    dim2 = dims(2);

    range = [options.ACQ.optim.bounds(dim1,:);options.ACQ.optim.bounds(dim2,:)];
    if nargin == 4
        v = 20; % number of contours
    elseif nargin ==5
        v = varargin{1}
    end
         
    if isfield(options.ACQ,'logprior') && ~isempty(options.ACQ.logprior)
        logprior = options.ACQ.logprior;
    else
        logprior = @(u)(0);
    end
        
    %% gathered data
    nX = evidence.nX;
    x = evidence.X(1:nX,dims);
    y = evidence.f(1:nX);
    if options.GP.log == 1
        y = exp(y);
    end
    y = y -logprior(x);
    
    %% estimates
    index = options.ACQ.maxRounds+1;
    thetaHat = evidence.thetaHat(index,:);
    while sum(isnan(thetaHat)>0)
        index = index-1;
        if index == 0
            break;
        end
        thetaHat = evidence.thetaHat(index,:);
    end
        
    uu1 = linspace(range(1,1),range(1,2),50)';
    uu2 = linspace(range(2,1),range(2,2),50)';
    [UU1 UU2]=meshgrid(uu1,uu2);
    U1 = UU1(:);
    U2 = UU2(:);
        
    u = repmat(thetaHat,length(U1),1);
    u(:,dim1)=U1;
    u(:,dim2)=U2;

   
    %% posterior mean and quantiles
    [postMean postVar_latent postVar] = post_mean_var(evidence,options.GP,u);
    if options.GP.log == 1
        postJmean =  exp(postMean+0.5*postVar);
    else
        postJmean = postMean;
    end
    postJmean = postJmean-logprior(u);
     
    
    if LOGSCALE
        postJmean = log(postJmean);
        y = log(y);
    end
    
    % maxi = max(postJmean(:));
    %mini = min(postJmean(:));
    %tmp = quantile(postJmean(:),0.4);
    %v1 = [linspace(mini,tmp,ceil(M/2))];
    %v2 = linspace(tmp,maxi,M-ceil(M/2));
    %v = [v1 v2(2:end)];
    %v = exp(linspace(log(min(evidence.f)),log(max(evidence.f)),M));

    contourf(UU1,UU2,reshape(postJmean,size(UU1)),v);
    hold on
    if WITH_SCATTER
        %scatter(x(:,1),x(:,2),[],y);
        plot(x(:,1),x(:,2),'k.','markersize',7);
    end
    plot(thetaHat(dim1),thetaHat(dim2),'rx','markersize',25)
    title('Mean')
    xlabel(['theta' num2str(dims(1))])
    ylabel(['theta' num2str(dims(2))])
    %colorbar
       
  
    