% --------------------------------------------------
%
% J = objective(theta,N,phiobs,n)
%
% theta    : L x 1 vector with parameters
% N        : number of data sets used to compute the average
% phiobs   : mean(yobs) 
% n        : number of samples in the observed data
%            y =  theta + randn(n,1);
% J        : L x 1 vector with objectives
%
% --------------------------------------------------

function J = objective(theta,N,phiobs,n)
    
    nTheta = length(theta);
    J = zeros(nTheta,1);
    
    for k=1:nTheta
       myTheta = theta(k);
       
        % Simulated summary statistics
        y = myTheta+randn(n,N);
        phi = mean(y,1);
       
        delta = bsxfun(@plus,phiobs,-phi).^2;
        J(k) = mean(delta)+eps;
    end
    