% --------------------------------------------------
%
% postAnalysisPlot(res,u,phiobs,n,fileName)
%
% res      : result structure containing the fields
%             res.evidence
%             res.options
%            with the results and options from the 
%            fastInference.m run
% u        : column vector of test points
% phiobs   : summary statistics of observed data
%            mean(yobs)
% n        : sample size of the observed data
% fileName : path and file name for export of the figures
%            if [] figures won't be exported
%
% Creates figures for comparison of the estimation 
% results with ground truth. 
%
% Assumes 'GP-synlik' setup, that is a GP-model and
% a synthetic-likelihood type of objective function.
%
% --------------------------------------------------


function postAnalysisPlot(res,u,phiobs,n,fileName)

procVar = 1;
priorMean = -10;
priorVar = 0.1;
p_prior = normpdf(u,priorMean,sqrt(priorVar));
trueLik = sqrt(n/(2*pi))*exp(-n/2*(phiobs-u).^2);

[p_post mu_n var_n] = truePostDistribution(priorMean,priorVar,phiobs,n,procVar,u);

figure; 
hold on
%h1=plot(u,p_prior,'g: ');
h1=plot(u,p_prior,'g-');
h2=plot(u,trueLik,'b-');
%h3=plot(u,p_post,'r-.');
h3=plot(u,p_post,'r-');

h = sqrt(1/n);
[logLik logLik_grad] = logLikApprox(res.evidence,res.options,u,h,['GP-synlik']);
likHat = exp(logLik);
likHat = likHat/max(likHat)*max(trueLik);
h4=plot(u,likHat,'m--');

postEstim = likHat.*p_prior;
postEstim = postEstim/max(postEstim)*max(p_post);
h5=plot(u,postEstim,'k--');


h6=plot(res.evidence.X,0*ones(size(res.evidence.X)),'k+');
set(gca,'xlim',[-12 5])
set(gca,'ylim',[0 2])

legend([h1 h2 h3 h4 h5 h6],'prior','likelihood','true posterior',...
       'estim likelihood','estim posterior','acq locations',...
       'location','northwest')
xlabel('\theta')

if ~isempty(fileName)
    myexportfig([fileName '_densities'])
end


figure
Jbar = ( (phiobs-u).^2+1/n);
h1=plot(u,Jbar,'r');
hold on
LOGSCALE = 0;
h2=quickPlot(res.evidence,res.options,LOGSCALE);
title('')
xlabel('\theta')
ylabel('neg log synthetic likelihood (arbitrary scale)')

set(gca,'xlim',[-5 4])
set(gca,'ylim',[-2 50])
legend([h1 h2],'true regression function',...
       'data','post mean','0.1, 0.9 quantiles')

if ~isempty(fileName)
    myexportfig([fileName '_objective'])
end


%%%%%%%%%%%%%%%%%%%%%%

function [p mu_n var_n] = truePostDistribution(priorMean,priorVar,obsMean,n,procVar,x)

var_n = 1/(1/priorVar+n/procVar);
mu_n = (priorMean/priorVar+n*obsMean/procVar)*var_n;

p = normpdf(x,mu_n,sqrt(var_n));