
% --------------------------------------------------
% 
% yCell = simulateRicker(theta,n,seed,S,id)
%
% theta: row vector of parameters, e.g. theta = [3.8 0.3 10];
% n      : sample size, e.g. n = 50;
% burnIn : burn-in sample size, e.g. burnin = 50;
% S      : number of data sets to sample, e.g. S=25
% id     : simulation id, e.g. '1'
%
% yCell: 1xS cell array of S data sets
%
% --------------------------------------------------


function yCell = simulateRicker(theta,n,burnIn,S,id)

PathToRFile = pwd;

e = randn(burnIn+n,S);
u = rand(1,n*S);

file_e = [id '_e.txt'];
file_u = [id '_u.txt'];

save(file_e,'-ascii','-double','e')
save(file_u,'-ascii','-double','u')

dims = [n S burnIn];
file_dims = [id '_rick_dims.txt'];
save(file_dims,'-ascii','-double','dims')

file_theta = [id '_theta.txt'];
save(file_theta,'-ascii','-double','theta')

% Run R
logFile = [id '_simRicker.log'];
Rcall = ['R CMD BATCH  -' id ' ' PathToRFile '/simulateRicker.R ' logFile];
system(Rcall);

% load
file_Y = [id '_Y.txt'];
if ~exist(file_Y,'file')
    keyboard;
end
Y = load(file_Y); % n x S

yCell = cell(1,S);
for k=1:S
    yCell{k}=Y(:,k); 
end

% clean up
delete(file_e); 
delete(file_u);
delete(file_dims);
delete(file_theta);
delete(logFile);
delete(file_Y);



