% --------------------------------------------------
%
%  val = synLogLik(theta,n,Yobs,burnIn,S,id);
%
% Evaluates the log synthetic likelihood of the 
% Ricker model at each row of theta.
% For each vector of parameters (row of theta), the
% same seed is used.
%
% -------------------------------------------------- 

function [val logdet]= synLogLik(theta,n,Yobs,burnIn,S,id);
 
    PathToRFile = pwd;
    
    e = randn(burnIn+n,S);
    u = rand(1,n*S);
    dims = [n S burnIn];
    
    file_e = [id '_obj_e.txt'];
    file_u = [id '_obj_u.txt'];
    file_Yobs = [id '_obj_Yobs.txt'];
    file_theta = [id '_obj_theta.txt'];
    file_dims = [id '_obj_dims.txt'];
     
    save(file_e,'-ascii','-double','e')
    save(file_u,'-ascii','-double','u')
    save(file_Yobs,'-ascii','-double','Yobs');
    save(file_theta,'-ascii','-double','theta');
    save(file_dims,'-ascii','-double','dims')

    % Run R
    logFile = [id '_obj.log'];
    Rcall = ['R CMD BATCH  -' id ' ' PathToRFile '/synLogLik.R ' logFile];
    system(Rcall);

    % load
    file_val = [id '_obj_val.txt'];
    file_detval = [id '_det_val.txt'];
    if ~exist(file_val,'file')
        keyboard;
    end
    val = load(file_val); % n x S
    half_logdet = load(file_detval);
    logdet = 2*half_logdet;
    
    % clean up
    delete(file_e); 
    delete(file_u);
    delete(file_Yobs); 
    delete(file_theta);
    delete(file_dims);
    delete(logFile);
    delete(file_val);
    delete(file_detval);
