function[observedstrains, observedcarriage, totalcoinfection, observedcoinfection, totalcarriage, intraRate, alienRate, theta] = ensemble_observations(parameters, sample_sizes, N_units, avg_DCC_size, global_strain_distribution, max_N_strains, simulation_time)

% This function simulates an ensemble of observations from independent
% local populations, with same model parameters. As an observation model,
% observed individuals are sampled using a vector of sample sizes, given by
% 'sample_sizes'. 

observedcoinfection = zeros(N_units,1);
observedcarriage = zeros(N_units,1);
observedstrains = zeros(max_N_strains,N_units);
state = initState(parameters, avg_DCC_size, max_N_strains);
theta = state.theta;
intraRate = state.intraRate;
alienRate = state.alienRate;


for  i=1:N_units
    
    DCC = individual_DCC_simulation(state, simulation_time, global_strain_distribution, avg_DCC_size, max_N_strains);
    sampled = randsample(avg_DCC_size, sample_sizes(i));
    observed = DCC(:, sampled);
    totalcarriage =sum(observed);
    observedcoinfection(i) = sum(totalcarriage>1)/sample_sizes(i);
    observedcarriage(i) =sum(totalcarriage>=1)/sample_sizes(i);
    observedstrains(:,i) = sum(observed,2);
    
end
totalcoinfection = (sum(sample_sizes.*observedcoinfection'))/sum(sample_sizes);             %over all populations, what is the prevalence of colonization
totalcarriage = (sum(sample_sizes.*observedcarriage'))/sum(sample_sizes);                   % -"- , the prevalence of co-infection.
end