% --------------------------------------------------
%
%  [all_post_mean CI all_hist lower_edges] = getMarginals(fileName,bounds,quants,M)
%
% Computes the marginal pdfs for the results stored in fileName
%
% --------------------------------------------------

function [all_post_mean CI all_hist lower_edges] = getMarginals(fileName,bounds,quants,M)

qL = quants(1);
qU = quants(2); 

nTheta = size(bounds,1);
postRes = load(fileName);
t = postRes.t;

allTheta = postRes.allTheta{t};    
weights = postRes.allWeights{t};

lower_edges = zeros(nTheta,M);
for k=1:nTheta
    lower_edges(k,:) = linspace(bounds(k,1),bounds(k,2),M);
end
all_post_mean = zeros(nTheta,1);
CI = zeros(nTheta,2);
all_hist = zeros(nTheta,M);

for k=1:nTheta
    mytheta = allTheta(k,:);
    [mu ~]=weighted_mean(mytheta,weights);
    LC = weighted_quantile(mytheta,weights,qL);
    UC = weighted_quantile(mytheta,weights,qU);
    
    all_post_mean(k) = mu;
    CI(k,:) = [LC UC];
    
    %[histw, vinterval] = weighted_hist(mytheta,weights,lower_edges(k,:));  
    histw = ksdensity(mytheta,lower_edges(k,:),'weights',weights);
    all_hist(k,:) = histw;
end
        