\name{MVN.check}
\alias{MVN.check}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Checking the multivariate normality approximation}
\description{
Let \code{S} be a matrix, each column of which is an i.i.d. observation on a random vector. This routine provides a 
graphical check of whether the distribution of the random vector is multivariate normal.
} %- end description

\usage{
MVN.check(S,s=NULL,cex.axis=1,cex.lab=1)
}
%- maybe also `usage' for other objects documented here.
\arguments{

\item{S}{a matrix, where each column is an i.i.d. observation on a random vector.}

\item{s}{a further observation on the random vector, not to be used in extimating the mean vector and covariance matrix.}

\item{cex.axis}{axis scale expansion factor.}

\item{cex.lab}{axis label expansion factor.}
}

\details{ The method is from section 7.5 of Krzanowski (1988). The replicate vectors are transformed to variables which should be univariate chi squared r.v.s with DoF given by the number of rows of \code{S}. An appropriate QQ-plot is produced, and the proportion of the data differing substantially from the ideal line is reported. Deviations at the right hand end of the plot indicate that the tail behaviour of the Normal approximation is poor: in the context of synthetic likelihood this is of little consequence.  

Secondly, \code{s} is transformed to a vector which should be i.i.d. N(0,1) under multivariate normality, and a QQ plot is produced. Unfortunately this approach is not very useful unless the dimension of \code{s} is rather large. In simulations, perfectly MVN data produce highly variable results, so that the approach lacks any real power. 
}
\value{ Mainly produces plots and prints output. Also an array indicating proportion of simulated statistics smaller than observed.
}
\references{

Krzanowski, W.J. (1988) Principles of Multivariate Analysis. Oxford.


\url{http://www.maths.bath.ac.uk/~sw283/}
}
\author{ Simon N. Wood \email{simon.wood@r-project.org}}


\seealso{  
blowfly, get.trans, robust.vcov
}

\examples{
par(mfrow=c(2,2))
library(MASS);library(sl)
V <- matrix(runif(13^2),13,13)
V <- V\%*\%t(V)
mu <- runif(13)
S <- t(mvrnorm(500,mu,V,tol=0))
s <- as.numeric(mvrnorm(1,mu,V,tol=0))
MVN.check(S,s)

}


\keyword{models} \keyword{smooth} \keyword{regression}%-- one or more ..





