% --------------------------------------------------
%
% evidence = init(options,obj)
% evidence = init(options,init_pt,init_f)
% 
% obj     : obj([theta_1;theta_2;...]) where theta_i 
%           is a row vector of parameters, returns
%           vector of objective values, one for each theta_i
% init_pt : #(points) x dimension of each point
% init_f :  #(points) x 1, or 
%           cell{1} = [#(points) x 1], 
%           cell{2} = cell with possible additional
%           info returned by the objective.
%
% --------------------------------------------------


function evidence = init(options, varargin)

%% Get some initial data
if nargin == 2
    obj = varargin{1};
         
    if options.GP.log == 1
        obj = @(u)(logobj(obj,u));
    end
    
    if isempty(options.IO.loadFile)
        if strcmp(options.ACQ.initMethod,'sobol') && options.GP.dim<=40
            tmp = sobol_dataset(options.GP.dim,options.ACQ.nInitPoints,0);
        else
            tmp = rand(options.GP.dim,options.ACQ.nInitPoints);
        end
        init_pt = bsxfun(@times,tmp,options.ACQ.optim.bounds(:,2)- ...
                         options.ACQ.optim.bounds(:,1));
        init_pt = bsxfun(@plus,init_pt,options.ACQ.optim.bounds(:,1))';
        init_f = obj(init_pt);
    else
        load(options.loadFile,'init_pt','init_f');
    end
elseif nargin == 3
    init_pt = varargin{1};
    init_f = varargin{2};
else
    error('init.m : Wrong number of input arguments')
end

if iscell(init_f)
    addiInfo = init_f{2};
    init_f = init_f{1};
else
    addiInfo = [];
end
      
%% ACQ
evidence.exploitCount = 0;


%%  GP 
d = options.GP.dim;

evidence.hyp = options.GP.hypInit;

% maximal size of covariance matrix: maxX x maxX
maxX = options.ACQ.maxX;
evidence.X = NaN(maxX, d);
evidence.f = NaN(maxX,1);
if ~isempty(addiInfo)
    evidence.container = cell(maxX,1); % for storing of possible additional output
end
evidence.prior_mean = NaN(maxX,1);

% set initial data
evidence.nX = length(init_f);
evidence.X(1:evidence.nX, :) = init_pt;    
evidence.f(1:evidence.nX,:) = init_f;
if ~isempty(addiInfo)
    for k=1:length(init_f)
        evidence.container{k} = addiInfo{k};
    end
end

[mini index]=min(init_f);
evidence.best_val = mini;
evidence.best_x = init_pt(index,:);
evidence.thetaHat = NaN(options.ACQ.maxRounds+1,options.GP.dim);
evidence.thetaHat(1,:) = evidence.best_x;

%% Fit hyperparameters

% fit hyperparameters
evidence = learn_hyper_param(evidence,options.GP,'init');
   






    
    
