% --------------------------------------------------
%
% Set all parameters used
%
% --------------------------------------------------

function options = setOptions_movie(saveFileTag,N)

%% Objective

% Number of replicated data sets per parameter
options.OBJ.N = N; 
options.OBJ.range = [0 100]; % rough guess of order of magnitude of obj
options.OBJ.logTransform = 0;

%% Acquisition

options.ACQ.maxAcqPerRound = ones(1,49);
options.ACQ.nPointsPerAcq = ones(1,49);
options.ACQ.nInitPoints = 1;
% each row corresponds to the [lower upper] bound of a parameter
options.ACQ.optim.bounds = [-5 5];

options.ACQ.acqSlack = 0;  
options.ACQ.logprior = '';


%% Filenames

options.IO.loadFile = ''; 
if isempty(saveFileTag)
    options.IO.tag = 'run01';
else
    options.IO.tag = saveFileTag;
end

acqstr = num2str(options.ACQ.acqSlack);
index = find(acqstr=='.');
if ~isempty(index)
    acqstr = acqstr(index+1:end);
end

options.IO.saveFile = ['../../matfiles/meanGauss1D/point_N' num2str(options.OBJ.N) '_slack' acqstr '_' options.IO.tag];

%% GP

% end-of-round optimization is only done if the number of acquired data points is above the indicated level
reqDataPoints = Inf; 

% after initialization, how long to wait till hyperparameters are updated
wait = 10;

% interval between two subsequent hyperparameter optimizations
interval = 10;


%% Details 

% Acquisition
if size(options.ACQ.optim.bounds,1)<=40
    options.ACQ.initMethod = 'sobol';
else
    options.ACQ.initMethod = 'rand'; 
end
maxPointsPerRound = options.ACQ.nPointsPerAcq.*options.ACQ.maxAcqPerRound;
options.ACQ.maxRounds = length(options.ACQ.maxAcqPerRound);
options.ACQ.maxX = options.ACQ.nInitPoints+sum(maxPointsPerRound);

options.ACQ.type = 'LCB'; %'EI','LCB'
options.ACQ.exploit.limitVar = (2*1e-3)^2;
options.ACQ.exploit.maxCount = 5;

options.ACQ.optim.useDIRECT = 1;
options.ACQ.optim.useCMAES = 1;
options.ACQ.optim.useFminsearch = 1;
options.ACQ.optim.useGradient = 1;

options.ACQ.optim.DIRECT.maxevals = 100;
options.ACQ.optim.DIRECT.maxits = 100;
    
options.ACQ.optim.CMAES.MaxFunEvals = 100;
options.ACQ.optim.CMAES.TolFun = 1e-8; 
    
options.ACQ.optim.FMINS.MaxFunEvals = 100;
    
options.ACQ.optim.TMP.method = 'lbfgs'; % 'sd', 'lbfgs'
options.ACQ.optim.TMP.maxIter = 100;
options.ACQ.optim.TMP.optTol = 1e-8;

% GP
if isfield(options.OBJ,'logTransform')
    options.GP.log = options.OBJ.logTransform;
else
    options.GP.log = 0;
end
options.GP.dim = size(options.ACQ.optim.bounds,1);
options.GP.cov = {@covSEard}; 
%options.GP.cov= {@covSEiso};
dCov = options.GP.dim;
%options.GP.mean = {@meanQuadratic};
options.GP.mean = {@meanConst}; 
options.GP.lik =  {@likGauss};

boundLength = options.ACQ.optim.bounds(:,2)-options.ACQ.optim.bounds(:,1);
objRange = options.OBJ.range(2)-options.OBJ.range(1);
options.GP.hypInit.cov = log([boundLength/3;objRange/10]);  % log[ ell1,ell2,..., sqrt(sf2) ], , covSEard, covSEiso                                                       

%options.GP.hypInit.mean = zeros(2*options.GP.dim+1,1); % quadratic, linear, const
options.GP.hypInit.mean = 0; % for @meanConst

options.GP.hypInit.lik = log(objRange/40); % log standard deviation

options.GP.optim.noBounds = 0;
%options.GP.optim.infMethod = @infExact;
options.GP.optim.infMethod = @infLOO;
options.GP.optim.maxUpdate = ceil(0.9*options.ACQ.maxX); % max number of data points after which
                                                          % we stop updating hyperparameters 

if options.GP.optim.noBounds
    options.GP.optim.init.maxIter = 10;
    options.GP.optim.middle.maxIter = 10;
    options.GP.optim.end.maxIter = 10;

    options.GP.optim.middle.wait = wait;      
    options.GP.optim.middle.interval = interval; 
    options.GP.optim.end.reqDataPoints = reqDataPoints;
else
    % large length-scale : long correlation, slowly varying function
     options.GP.optim.Lbounds.cov = [log(0.1*boundLength);log(objRange/10)]; % length-scales, log( signal std ) 
     options.GP.optim.Ubounds.cov = [log(boundLength); log(2*objRange)];
    
     options.GP.optim.Lbounds.lik = log(objRange/100); % log noise standard deviation, 
     options.GP.optim.Ubounds.lik = log(objRange);
    
    % @meanQuadratic, quadratic, linear, const
    %options.GP.optim.Lbounds.mean = [zeros(options.GP.dim,1);-1000*ones(options.GP.dim,1);options.OBJ.range(1)-1000*max(1,abs(options.OBJ.range(1)))];
    %options.GP.optim.Ubounds.mean = [1000*ones(options.GP.dim,1);1000*ones(options.GP.dim,1);options.OBJ.range(2)+1000*max(1,abs(options.OBJ.range(2)))];
    
    % @meanConst
    options.GP.optim.Lbounds.mean = -1000;
    options.GP.optim.Ubounds.mean = 1000;
    
    %** for init
    options.GP.optim.init.useDIRECT = 0;
    options.GP.optim.init.useCMAES = 0;
    options.GP.optim.init.useFminsearch = 0;
    options.GP.optim.init.useGradient = 0;
    
    nb = 0;
    options.GP.optim.init.DIRECT.maxevals = nb; % max number of function evals  
    options.GP.optim.init.DIRECT.maxits = nb; % max number of iterations    

    options.GP.optim.init.CMAES.MaxFunEvals = nb;
    options.GP.optim.init.CMAES.TolFun = 1e-8; 
        
    options.GP.optim.init.FMINS.MaxFunEvals = nb;
        
    options.GP.optim.init.TMP.method = 'lbfgs'; % 'sd', 'lbfgs'
    options.GP.optim.init.TMP.maxIter = nb;
    options.GP.optim.init.TMP.optTol = 1e-8;
        
    %** during BO
    options.GP.optim.middle.useDIRECT = 0;
    options.GP.optim.middle.useCMAES = 0;
    options.GP.optim.middle.useFminsearch = 0;
    options.GP.optim.middle.useGradient = 1;
    
    nb = 1;
    options.GP.optim.middle.DIRECT.maxevals = nb; % max number of function evals  
    options.GP.optim.middle.DIRECT.maxits = nb; % max number of iterations    

    options.GP.optim.middle.CMAES.MaxFunEvals = nb;
    options.GP.optim.middle.CMAES.TolFun = 1e-8; 
        
    options.GP.optim.middle.FMINS.MaxFunEvals = nb;
        
    options.GP.optim.middle.TMP.method = 'lbfgs'; % 'sd', 'lbfgs'
    options.GP.optim.middle.TMP.maxIter = nb;
    options.GP.optim.middle.TMP.optTol = 1e-8;
 
    options.GP.optim.middle.wait = wait;          
    options.GP.optim.middle.interval = interval;  
    
    %** at the end of each round if enough data has been acquired
    options.GP.optim.end.reqDataPoints = reqDataPoints;
    options.GP.optim.end.useDIRECT = 1;
    options.GP.optim.end.useCMAES = 1;
    options.GP.optim.end.useFminsearch = 1;
    options.GP.optim.end.useGradient = 1;

    options.GP.optim.end.DIRECT.maxevals = 10;
    options.GP.optim.end.DIRECT.maxits = 10;
    
    options.GP.optim.end.CMAES.MaxFunEvals = 10;
    options.GP.optim.end.CMAES.TolFun = 1e-8; 
    
    options.GP.optim.end.FMINS.MaxFunEvals = 10;
    
    options.GP.optim.end.TMP.method = 'lbfgs'; % 'sd', 'lbfgs'
    options.GP.optim.end.TMP.maxIter = 10;
    options.GP.optim.end.TMP.optTol = 1e-8;
    
end


% Optimization of mean function

 options.ESTIM.optim.useDIRECT = 1;
 options.ESTIM.optim.useCMAES = 1;
 options.ESTIM.optim.useFminsearch = 1;
 options.ESTIM.optim.useGradient = 1;
 
 options.ESTIM.optim.DIRECT.maxevals = 100; % max number of function evals  
 options.ESTIM.optim.DIRECT.maxits = 50; % max number of iterations
 
 options.ESTIM.optim.CMAES.MaxFunEvals = 100;
 options.ESTIM.optim.CMAES.TolFun = 1e-8; 
 
 options.ESTIM.optim.FMINS.MaxFunEvals = 100;
 
 options.ESTIM.optim.TMP.method = 'lbfgs'; % 'sd', 'lbfgs'
 options.ESTIM.optim.TMP.maxIter = 100;
 options.ESTIM.optim.TMP.optTol = 1e-8;
 
 
 %% Keep a record of the options used
 copyfile([mfilename '.m'],[options.IO.saveFile '_options.m']);
    
    
 
 